From: Xavier Deguillard <xdeguill...@vmware.com>

Currently, the balloon driver would fail to run if memory is greater
than 16TB of vRAM. Previous patches have already converted the balloon
target and size to 64-bit, so all that is left to do add is to avoid
asserting memory is smaller than 16TB if the hypervisor supports 64-bits
target.

The driver advertises a new capability VMW_BALLOON_64_BITS_TARGET.
Hypervisors that support 16TB of memory or more will report that this
capability is enabled.

Signed-off-by: Xavier Deguillard <xdeguill...@vmware.com>
Signed-off-by: Nadav Amit <na...@vmware.com>
---
 drivers/misc/vmw_balloon.c | 18 +++++++++++++++---
 1 file changed, 15 insertions(+), 3 deletions(-)

diff --git a/drivers/misc/vmw_balloon.c b/drivers/misc/vmw_balloon.c
index 27cac1f98e1e..372282de649e 100644
--- a/drivers/misc/vmw_balloon.c
+++ b/drivers/misc/vmw_balloon.c
@@ -59,13 +59,24 @@ enum vmwballoon_capabilities {
        VMW_BALLOON_BATCHED_CMDS                = (1 << 2),
        VMW_BALLOON_BATCHED_2M_CMDS             = (1 << 3),
        VMW_BALLOON_SIGNALLED_WAKEUP_CMD        = (1 << 4),
+       VMW_BALLOON_64_BIT_TARGET               = (1 << 5)
 };
 
-#define VMW_BALLOON_CAPABILITIES       (VMW_BALLOON_BASIC_CMDS \
+#define VMW_BALLOON_CAPABILITIES_COMMON        (VMW_BALLOON_BASIC_CMDS \
                                        | VMW_BALLOON_BATCHED_CMDS \
                                        | VMW_BALLOON_BATCHED_2M_CMDS \
                                        | VMW_BALLOON_SIGNALLED_WAKEUP_CMD)
 
+/*
+ * 64-bit targets are only supported in 64-bit
+ */
+#ifdef CONFIG_64BIT
+#define VMW_BALLOON_CAPABILITIES       (VMW_BALLOON_CAPABILITIES_COMMON \
+                                       | VMW_BALLOON_64_BIT_TARGET)
+#else
+#define VMW_BALLOON_CAPABILITIES       VMW_BALLOON_CAPABILITIES_COMMON
+#endif
+
 enum vmballoon_page_size_type {
        VMW_BALLOON_4K_PAGE,
        VMW_BALLOON_2M_PAGE,
@@ -549,8 +560,9 @@ static int vmballoon_send_get_target(struct vmballoon *b)
 
        limit = totalram_pages;
 
-       /* Ensure limit fits in 32-bits */
-       if (limit != (u32)limit)
+       /* Ensure limit fits in 32-bits if 64-bit targets are not supported */
+       if (!(b->capabilities & VMW_BALLOON_64_BIT_TARGET) &&
+           limit != (u32)limit)
                return -EINVAL;
 
        status = vmballoon_cmd(b, VMW_BALLOON_CMD_GET_TARGET, limit, 0);
-- 
2.17.1

Reply via email to